var SocketIORemoteEndpoint,
  slice = [].slice;

SocketIORemoteEndpoint = (function() {
  function SocketIORemoteEndpoint() {}

  SocketIORemoteEndpoint.prototype.initialize = function(options, callback) {
    if (options == null) {
      options = {};
    }
    if (callback == null) {
      callback = function() {};
    }
    if (options.ioInstance) {
      this._io = options.ioInstance;
    } else {
      this._io = require('socket.io')();
      this._io.listen(3000);
    }
    if (options.rpcRequestMiddleware) {
      this._rpcRequestMiddleware = options.rpcRequestMiddleware;
    } else {
      this._rpcRequestMiddleware = function(request, socket, callback) {
        return callback();
      };
    }
    this._io.sockets.on('connection', (function(_this) {
      return function(socket) {
        socket.on('RPC_Request', function(rpcRequest) {
          var emitRpcResponse;
          emitRpcResponse = function(error, response) {
            var rpcId;
            rpcId = rpcRequest.rpcId;
            return socket.emit('RPC_Response', {
              rpcId: rpcId,
              err: error,
              data: response
            });
          };
          return _this._rpcRequestMiddleware(rpcRequest, socket, function() {
            return _this._handleRPCRequest(rpcRequest, emitRpcResponse);
          });
        });
        socket.on('JoinRoom', function(roomName) {
          return socket.join(roomName);
        });
        return socket.on('LeaveRoom', function(roomName) {
          return socket.leave(roomName);
        });
      };
    })(this));
    return callback();
  };

  SocketIORemoteEndpoint.prototype.close = function() {
    return this._io.close();
  };

  SocketIORemoteEndpoint.prototype.setRPCHandler = function(_handleRPCRequest) {
    this._handleRPCRequest = _handleRPCRequest;
  };

  SocketIORemoteEndpoint.prototype.publish = function() {
    var aggregateId, arg, context, domainEventName, fullEventName, i, payload;
    context = arguments[0], arg = 3 <= arguments.length ? slice.call(arguments, 1, i = arguments.length - 1) : (i = 1, []), payload = arguments[i++];
    domainEventName = arg[0], aggregateId = arg[1];
    fullEventName = this._getFullEventName(context, domainEventName, aggregateId);
    return this._io.to(fullEventName).emit(fullEventName, payload);
  };

  SocketIORemoteEndpoint.prototype._getFullEventName = function(context, domainEventName, aggregateId) {
    var fullEventName;
    fullEventName = context;
    if (domainEventName) {
      fullEventName += "/" + domainEventName;
    }
    if (aggregateId) {
      fullEventName += "/" + aggregateId;
    }
    return fullEventName;
  };

  return SocketIORemoteEndpoint;

})();

module.exports = new SocketIORemoteEndpoint;
